package activerecorder;

import java.awt.AWTException;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import activerecorder.CTRLFrame;

public class ActiveRecorder extends Thread{

	private boolean dorun;
	private File dir;
	private Rectangle rect;
	private static int pause;
	private BufferedImage lastwritten;
	private int numberOfPixelsToDecideUpdated;
	
	public ActiveRecorder(File d, Rectangle r, int timeout,int pix) {
		dir=d;
		rect=r;
		pause=timeout;
		numberOfPixelsToDecideUpdated=pix;
	}

	public static void prepare(){
		JFileChooser fc=new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fc.showSaveDialog(null);
		File outfile=fc.getSelectedFile();
		if(outfile==null){System.exit(0);}
		
		String areainput=(String)JOptionPane.showInputDialog(null,"Area to record (x1,y1,x2,y2)","0,0,1680,1050");
		StringTokenizer st=new StringTokenizer(areainput,",");
		int x1=Integer.parseInt(st.nextToken());
		int y1=Integer.parseInt(st.nextToken());
		int x2=Integer.parseInt(st.nextToken());
		int y2=Integer.parseInt(st.nextToken());
		
		String timeinput=(String)JOptionPane.showInputDialog(null,"Time to wait before screenshots (milliseconds)","1000");
		int timeout=Integer.parseInt(timeinput);
		
		String pixelinput=(String)JOptionPane.showInputDialog(null,"Number of pixels to be updated for screen to be considered updated","2048");
		int pix=Integer.parseInt(pixelinput);
		
		Rectangle r=new Rectangle(x1,y1,x2-x1,y2-y1);
		
		ActiveRecorder ar=new ActiveRecorder(outfile,r,timeout,pix);
		CTRLFrame cf=new CTRLFrame(ar);
		System.out.println("Prepared");
	}
	
	public synchronized void run(){
		System.out.println("Start");
		Robot r;
		try{
			r = new Robot();

		ZipOutputStream zipo;
		try{
			zipo = new ZipOutputStream(new FileOutputStream(dir));

		dorun=true;
		try{
		try{
			while(dorun){
				BufferedImage bim=r.createScreenCapture(rect);
				String time=""+System.currentTimeMillis();
				
				if(updated(bim)){
					lastwritten=bim;
					zipo.putNextEntry(new ZipEntry(time+".png"));
					ImageIO.write(bim,"png",zipo);
				}
				wait(pause);
			}
			zipo.close();
	}catch(InterruptedException ee){} 
	}catch(IOException ee){}
	}catch (FileNotFoundException e){}
	}catch (AWTException e1){}
		System.exit(0);
	}
	
	private boolean updated(BufferedImage bim){
		boolean upd=false;
		if(lastwritten==null){upd=true;}
		else{
			int chp=0;
	        for (int x = 0; x < bim.getWidth(); x++) {
	            for (int y = 0; y < bim.getHeight(); y++) {
	                if (bim.getRGB(x, y) != lastwritten.getRGB(x, y)){
	                    chp++;
	                    if(chp>numberOfPixelsToDecideUpdated) break;
	                }
	            }
	        }
	            if(chp>numberOfPixelsToDecideUpdated) upd=true;
		}
		return upd;
	}
	
	public synchronized void endrec(){
		dorun=false;
	}
	
}
